@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2022 Salvador E. Tropea
-# Copyright (c) 2022 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2022-2023 Salvador E. Tropea
+# Copyright (c) 2022-2023 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 """
@@ -60,36 +60,58 @@ Dependencies:
     debian: imagemagick
     arch: imagemagick
     extra_arch: ['gsfonts']
-  - name: PcbDraw
-    # 0.9.0 implements KiCad 6 support
-    # 0.9.0.3 Fixes KiCad 5 problems
-    version: 0.9.0.3
-    max_version: 1.0
-    github: INTI-CMNB/pcbdraw
-    pypi: PcbDraw
-    downloader: pytool
-    comments: 'Currently the upstream version is broken, please use the mentioned fork'
   - name: KiCost
     github: hildogjr/KiCost
     pypi: KiCost
     downloader: pytool
+  - name: LXML
+    python_module: true
+    debian: python3-lxml
+    arch: python-lxml
+    downloader: python
+  - name: KiKit
+    github: yaqwsx/KiKit
+    pypi: KiKit
+    downloader: pytool
+  - from: KiKit
+    role: Separate multiboard projects
+  - name: Xvfbwrapper
+    python_module: true
+    debian: python3-xvfbwrapper
+    arch: python-xvfbwrapper
+    downloader: python
+  - name: Xvfb
+    url: https://www.x.org
+    command: xvfb-run
+    debian: xvfb
+    arch: xorg-server-xvfb
+    no_cmd_line_version: true
+  - name: Bash
+    url: https://www.gnu.org/software/bash/
+    debian: bash
+    arch: bash
+  - name: Blender
+    url: https://www.blender.org/
+    debian: blender
+    arch: blender
 """
-import importlib
-import os
-import re
-import subprocess
-import requests
-import platform
-import io
-import tarfile
-import stat
-import json
-import fnmatch
-import site
-from sys import exit, stdout, modules
-from shutil import which, rmtree, move
-from math import ceil
 from copy import deepcopy
+import fnmatch
+import importlib
+import io
+import json
+from math import ceil
+import os
+import platform
+import re
+import requests
+from shutil import which, rmtree, move
+import site
+import stat
+import subprocess
+from sys import exit, stdout, modules
+import tarfile
+from time import sleep
 from .misc import MISSING_TOOL, TRY_INSTALL_CHECK, W_DOWNTOOL, W_MISSTOOL, USER_AGENT, version_str2tuple
 from .gs import GS
 from .registrable import RegDependency
@@ -135,6 +157,21 @@ def end_show_progress():
     stdout.flush()
 
 
+def get_request(url):
+    retry = 4
+    while retry:
+        r = requests.get(url, timeout=20, allow_redirects=True, headers={'User-Agent': USER_AGENT})
+        if r.status_code == 200:
+            return r
+        if r.status_code == 403:
+            # GitHub returns 403 randomly (sturated?)
+            sleep(1 << (4-retry))
+            retry -= 1
+        else:
+            return r
+    return r
+
+
 def download(url, progress=True):
     logger.debug('- Trying to download '+url)
     r = requests.get(url, allow_redirects=True, headers={'User-Agent': USER_AGENT}, timeout=20, stream=True)
@@ -179,7 +216,7 @@ def try_download_tar_ball(dep, url, name, name_in_tar=None):
         name_in_tar = name
     content = download(url)
     if content is None:
-        return None
+        return None, None
     # Try to extract the binary
     dest_file = None
     try:
@@ -190,13 +227,13 @@ def try_download_tar_ball(dep, url, name, name_in_tar=None):
                 dest_file = write_executable(name, tar.extractfile(entry).read())
     except Exception as e:
         logger.debug('- Failed to extract {}'.format(e))
-        return None
+        return None, None
     # Is this usable?
-    cmd = check_tool_binary_version(dest_file, dep, no_cache=True)
+    cmd, ver = check_tool_binary_version(dest_file, dep, no_cache=True)
     if cmd is None:
-        return None
+        return None, None
     # logger.warning(W_DOWNTOOL+'Using downloaded `{}` tool, please visit {} for details'.format(name, dep.url))
-    return cmd
+    return cmd, ver
 
 
 def untar(data):
@@ -280,10 +317,10 @@ def pytool_downloader(dep, system, plat):
     # Check if we have a github repo as download page
     logger.debug('- Download URL: '+str(dep.url_down))
     if not dep.url_down:
-        return None
+        return None, None
     res = re.match(r'^https://github.com/([^/]+)/([^/]+)/', dep.url_down)
     if res is None:
-        return None
+        return None, None
     user = res.group(1)
     prj = res.group(2)
     logger.debugl(2, '- GitHub repo: {}/{}'.format(user, prj))
@@ -291,27 +328,27 @@ def pytool_downloader(dep, system, plat):
     # Check if we have pip and wheel
     pip_command = check_pip()
     if pip_command is None:
-        return None
+        return None, None
     # Look for the last release
     data = download(url, progress=False)
     if data is None:
-        return None
+        return None, None
     try:
         data = json.loads(data)
         logger.debugl(4, 'Release information: {}'.format(data))
         url = data['tarball_url']
     except Exception as e:
         logger.debug('- Failed to find a download ({})'.format(e))
-        return None
+        return None, None
     logger.debugl(2, '- Tarball: '+url)
     # Download and uncompress the tarball
     dest = untar(download(url))
     if dest is None:
-        return None
+        return None, None
     logger.debugl(2, '- Uncompressed tarball to: '+dest)
     # Try to pip install it
     if not pip_install(pip_command, dest=dest):
-        return None
+        return None, None
     rmtree(dest)
     # Check it was successful
     return check_tool_binary_version(os.path.join(site.USER_BASE, 'bin', dep.command), dep, no_cache=True)
@@ -334,13 +371,13 @@ def git_downloader(dep, system, plat):
     # arm, arm64, mips64el and mipsel are also there, just not implemented
     if system != 'Linux' or not plat.startswith('x86_'):
         logger.debug('- No binary for this system')
-        return None
+        return None, None
     # Try to download it
     arch = 'amd64' if plat == 'x86_64' else 'i386'
     url = 'https://github.com/EXALAB/git-static/raw/master/output/'+arch+'/bin/git'
     content = download(url)
     if content is None:
-        return None
+        return None, None
     dest_bin = write_executable(dep.command+'.real', content.replace(b'/root/output', b'/tmp/kibogit'))
     # Now create the wrapper
     git_real = dest_bin
@@ -361,31 +398,31 @@ def convert_downloader(dep, system, plat):
     # Currently only for Linux x86_64
     if system != 'Linux' or plat != 'x86_64':
         logger.debug('- No binary for this system')
-        return None
+        return None, None
     # Get the download page
     content = download(dep.url_down)
     if content is None:
-        return None
+        return None, None
     # Look for the URL
     res = re.search(r'href\s*=\s*"([^"]+)">magick<', content.decode())
     if not res:
         logger.debug('- No `magick` download')
-        return None
+        return None, None
     url = res.group(1)
     # Get the binary
     content = download(url)
     if content is None:
-        return None
+        return None, None
     # Can we run the AppImage?
     dest_bin = write_executable(dep.command, content)
-    cmd = check_tool_binary_version(dest_bin, dep, no_cache=True)
+    cmd, ver = check_tool_binary_version(dest_bin, dep, no_cache=True)
     if cmd is not None:
         logger.warning(W_DOWNTOOL+'Using downloaded `{}` tool, please visit {} for details'.format(dep.name, dep.url))
-        return cmd
+        return cmd, ver
     # Was because we don't have FUSE support
     if not ('libfuse.so' in last_stderr or 'FUSE' in last_stderr or last_stderr.startswith('fuse')):
         logger.debug('- Unknown fail reason: `{}`'.format(last_stderr))
-        return None
+        return None, None
     # Uncompress it
     unc_dir = os.path.join(home_bin, 'squashfs-root')
     if os.path.isdir(unc_dir):
@@ -396,16 +433,16 @@ def convert_downloader(dep, system, plat):
         res_run = subprocess.run(cmd, check=True, capture_output=True, cwd=home_bin)
     except Exception as e:
         logger.debug('- Failed to execute `{}` ({})'.format(cmd[0], e))
-        return None
+        return None, None
     if not os.path.isdir(unc_dir):
         logger.debug('- Failed to uncompress `{}` ({})'.format(cmd[0], res_run.stderr.decode()))
-        return None
+        return None, None
     # Now copy the important stuff
     # Binaries
     src_dir, _, bins = next(os.walk(os.path.join(unc_dir, 'usr', 'bin')))
     if not len(bins):
         logger.debug('- No binaries found after extracting {}'.format(dest_bin))
-        return None
+        return None, None
     for f in bins:
         dst_file = os.path.join(home_bin, f)
         if os.path.isfile(dst_file):
@@ -415,7 +452,7 @@ def convert_downloader(dep, system, plat):
     src_dir = os.path.join(unc_dir, 'usr', 'lib')
     if not os.path.isdir(src_dir):
         logger.debug('- No libraries found after extracting {}'.format(dest_bin))
-        return None
+        return None, None
     dst_dir = os.path.join(home_bin, '..', 'lib', 'ImageMagick')
     if os.path.isdir(dst_dir):
         rmtree(dst_dir)
@@ -426,7 +463,7 @@ def convert_downloader(dep, system, plat):
     src_dir, dirs, _ = next(os.walk(os.path.join(unc_dir, 'usr', 'etc')))
     if len(dirs) != 1:
         logger.debug('- More than one config dir found {}'.format(dirs))
-        return None
+        return None, None
     src_dir = os.path.join(src_dir, dirs[0])
     dst_dir = os.path.join(home_bin, '..', 'etc')
     os.makedirs(dst_dir, exist_ok=True)
@@ -456,13 +493,13 @@ def gs_downloader(dep, system, plat):
     # Currently only for Linux x86
     if system != 'Linux' or not plat.startswith('x86_'):
         logger.debug('- No binary for this system')
-        return None
+        return None, None
     # Get the download page
     url = 'https://api.github.com/repos/ArtifexSoftware/ghostpdl-downloads/releases/latest'
-    r = requests.get(url, allow_redirects=True)
+    r = get_request(url)
     if r.status_code != 200:
         logger.debug('- Failed to download `{}`'.format(dep.url_down))
-        return None
+        return None, None
     # Look for the valid tarball
     arch = 'x86_64' if plat == 'x86_64' else 'x86'
     url = None
@@ -476,28 +513,28 @@ def gs_downloader(dep, system, plat):
         logger.debug('- Failed to find a download ({})'.format(e))
     if url is None:
         logger.debug('- No suitable binary')
-        return None
+        return None, None
     # Try to download it
-    res = try_download_tar_ball(dep, url, 'gs', 'ghostscript-*/gs*')
+    res, ver = try_download_tar_ball(dep, url, 'gs', 'ghostscript-*/gs*')
     if res is not None:
         short_gs = res
         long_gs = res[:-2]+'ghostscript'
         if not os.path.isfile(long_gs):
             os.symlink(short_gs, long_gs)
-    return res
+    return res, ver
 
 
 def rsvg_downloader(dep, system, plat):
     # Currently only for Linux x86_64
     if system != 'Linux' or plat != 'x86_64':
         logger.debug('- No binary for this system')
-        return None
+        return None, None
     # Get the download page
     url = 'https://api.github.com/repos/set-soft/rsvg-convert-aws-lambda-binary/releases/latest'
-    r = requests.get(url, allow_redirects=True)
+    r = get_request(url)
     if r.status_code != 200:
         logger.debug('- Failed to download `{}`'.format(dep.url_down))
-        return None
+        return None, None
     # Look for the valid tarball
     url = None
     try:
@@ -509,21 +546,21 @@ def rsvg_downloader(dep, system, plat):
         logger.debug('- Failed to find a download ({})'.format(e))
     if url is None:
         logger.debug('- No suitable binary')
-        return None
+        return None, None
     # Try to download it
     return try_download_tar_ball(dep, url, 'rsvg-convert')
 
 
 def rar_downloader(dep, system, plat):
     # Get the download page
-    r = requests.get(dep.url_down, allow_redirects=True)
+    r = get_request(dep.url_down)
     if r.status_code != 200:
         logger.debug('- Failed to download `{}`'.format(dep.url_down))
-        return None
+        return None, None
     # Try to figure out the right package
     OSs = {'Linux': 'rarlinux', 'Darwin': 'rarmacos'}
     if system not in OSs:
-        return None
+        return None, None
     name = OSs[system]
     if plat == 'arm64':
         name += '-arm'
@@ -532,10 +569,10 @@ def rar_downloader(dep, system, plat):
     elif plat == 'x86_32':
         name += '-x32'
     else:
-        return None
+        return None, None
     res = re.search('href="([^"]+{}[^"]+)"'.format(name), r.content.decode())
     if not res:
-        return None
+        return None, None
     # Try to download it
     return try_download_tar_ball(dep, dep.url+res.group(1), 'rar', name_in_tar='rar/rar')
 
@@ -563,6 +600,9 @@ def run_command(cmd, only_first_line=False, pre_ver_text=None, no_err_2=False):
         return None
     last_stderr = res_run.stderr.decode()
     res = res_run.stdout.decode().strip()
+    if len(res) == 0 and len(last_stderr) != 0:
+        # Ok, yes, OpenSCAD prints its version to stderr!!!
+        res = last_stderr
     if only_first_line:
         res = res.split('\n')[0]
     pre_vers = (cmd[0]+' version ', cmd[0]+' ', pre_ver_text)
@@ -583,7 +623,7 @@ def check_tool_binary_version(full_name, dep, no_cache=False):
     if dep.no_cmd_line_version:
         # No way to know the version, assume we can use it
         logger.debugl(2, "- This tool doesn't have a version option")
-        return full_name
+        return full_name, None
     # Do we need a particular version?
     needs = (0, 0, 0)
     for r in dep.roles:
@@ -606,7 +646,7 @@ def check_tool_binary_version(full_name, dep, no_cache=False):
         binary_tools_cache[full_name] = version
         logger.debugl(2, '- Found version {}'.format(version))
     version_check_fail = version is None or version < needs
-    return None if version_check_fail else full_name
+    return None if version_check_fail else full_name, version
 
 
 def check_tool_binary_system(dep):
@@ -616,7 +656,7 @@ def check_tool_binary_system(dep):
     else:
         full_name = which(dep.command)
     if full_name is None:
-        return None
+        return None, None
     return check_tool_binary_version(full_name, dep)
 
 
@@ -628,14 +668,14 @@ def check_tool_binary_local(dep):
     logger.debugl(2, '- Looking for tool `{}` at user level'.format(dep.command))
     home = os.environ.get('HOME') or os.environ.get('username')
     if home is None:
-        return None
+        return None, None
     full_name = os.path.join(home_bin, dep.command)
     if not os.path.isfile(full_name) or not os.access(full_name, os.X_OK):
-        return None
-    cmd = check_tool_binary_version(full_name, dep)
+        return None, None
+    cmd, ver = check_tool_binary_version(full_name, dep)
     if cmd is not None:
         using_downloaded(dep)
-    return cmd
+    return cmd, ver
 
 
 def check_tool_binary_python(dep):
@@ -643,13 +683,13 @@ def check_tool_binary_python(dep):
     logger.debugl(2, '- Looking for tool `{}` at Python user site ({})'.format(dep.command, base))
     full_name = os.path.join(base, dep.command)
     if not os.path.isfile(full_name) or not os.access(full_name, os.X_OK):
-        return None
+        return None, None
     return check_tool_binary_version(full_name, dep)
 
 
 def try_download_tool_binary(dep):
     if dep.downloader is None or home_bin is None:
-        return None
+        return None, None
     logger.info('- Trying to download {} ({})'.format(dep.name, dep.url_down))
     res = None
     # Determine the platform
@@ -667,28 +707,28 @@ def try_download_tool_binary(dep):
     # res = dep.downloader(dep, system, plat)
     # return res
     try:
-        res = dep.downloader(dep, system, plat)
+        res, ver = dep.downloader(dep, system, plat)
         if res:
             using_downloaded(dep)
     except Exception as e:
         logger.error('- Failed to download {}: {}'.format(dep.name, e))
-    return res
+    return res, ver
 
 
 def check_tool_binary(dep):
     logger.debugl(2, '- Checking binary tool {}'.format(dep.name))
-    cmd = check_tool_binary_system(dep)
+    cmd, ver = check_tool_binary_system(dep)
     if cmd is not None:
-        return cmd
-    cmd = check_tool_binary_python(dep)
+        return cmd, ver
+    cmd, ver = check_tool_binary_python(dep)
     if cmd is not None:
-        return cmd
-    cmd = check_tool_binary_local(dep)
+        return cmd, ver
+    cmd, ver = check_tool_binary_local(dep)
     if cmd is not None:
-        return cmd
+        return cmd, ver
     global disable_auto_download
     if disable_auto_download:
-        return None
+        return None, None
     return try_download_tool_binary(dep)
 
 
@@ -713,7 +753,7 @@ def check_tool_python_version(mod, dep):
         version = 'Ok'
     logger.debugl(2, '- Found version {}'.format(version))
     version_check_fail = version != 'Ok' and version < needs
-    return None if version_check_fail else mod
+    return None if version_check_fail else mod, version
 
 
 def check_tool_python(dep, reload=False):
@@ -727,21 +767,21 @@ def check_tool_python(dep, reload=False):
     # Not installed, try to download it
     global disable_auto_download
     if disable_auto_download or not python_downloader(dep):
-        return None
+        return None, None
     # Check we can use it
     try:
         importlib.invalidate_caches()
         mod = importlib.import_module(dep.module_name)
         if mod.__file__ is None:
             logger.error(mod)
-            return None
-        res = check_tool_python_version(mod, dep)
+            return None, None
+        res, ver = check_tool_python_version(mod, dep)
         if res is not None and reload:
             res = importlib.reload(reload)
-        return res
+        return res, ver
     except ModuleNotFoundError:
         pass
-    return None
+    return None, None
 
 
 def do_log_err(msg, fatal):
@@ -780,14 +820,14 @@ def get_dep_data(context, dep):
     return used_deps[context+':'+dep.lower()]
 
 
-def check_tool_dep(context, dep, fatal=False):
+def check_tool_dep_get_ver(context, dep, fatal=False):
     dep = get_dep_data(context, dep)
     logger.debug('Starting tool check for {}'.format(dep.name))
     if dep.is_python:
-        cmd = check_tool_python(dep)
+        cmd, ver = check_tool_python(dep)
         type = 'python module'
     else:
-        cmd = check_tool_binary(dep)
+        cmd, ver = check_tool_binary(dep)
         type = 'command'
     logger.debug('- Returning `{}`'.format(cmd))
     if cmd is None:
@@ -818,11 +858,17 @@ def check_tool_dep(context, dep, fatal=False):
         do_log_err(TRY_INSTALL_CHECK, fatal)
         if fatal:
             exit(MISSING_TOOL)
+    return cmd, ver
+
+
+def check_tool_dep(context, dep, fatal=False):
+    cmd, ver = check_tool_dep_get_ver(context, dep, fatal)
     return cmd
 
 
 # Avoid circular deps. Optionable can use it.
 GS.check_tool_dep = check_tool_dep
+GS.check_tool_dep_get_ver = check_tool_dep_get_ver
 
 
 class ToolDependencyRole(object):
@@ -935,6 +981,7 @@ def register_dep(context, dep):
     command = dep.get('command', None)
     help_option = dep.get('help_option', None)
     pypi_name = dep.get('pypi', None)
+    no_cmd_line_version = dep.get('no_cmd_line_version', False)
     no_cmd_line_version_old = dep.get('no_cmd_line_version_old', False)
     downloader_str = downloader = dep.get('downloader', None)
     if downloader:
@@ -947,7 +994,7 @@ def register_dep(context, dep):
                         extra_deb=extra_deb, is_python=is_python, module_name=module_name, plugin_dirs=plugin_dirs,
                         command=command, help_option=help_option, pypi_name=pypi_name,
                         no_cmd_line_version_old=no_cmd_line_version_old, downloader=downloader, arch=arch,
-                        extra_arch=extra_arch, tests=tests)
+                        extra_arch=extra_arch, tests=tests, no_cmd_line_version=no_cmd_line_version)
     # Extra comments
     comments = dep.get('comments', [])
     if isinstance(comments, str):
