@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Salvador E. Tropea
-# Copyright (c) 2020 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2023 Salvador E. Tropea
+# Copyright (c) 2020-2023 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 # Description: Implements the KiBoM and IBoM filters.
@@ -67,11 +67,15 @@ class Generic(BaseFilter):  # noqa: F821
             self.exclude_all_hash_ref = False
             """ Exclude all components with a reference starting with # """
             self.exclude_virtual = False
-            """ KiCad 5: exclude components marked as virtual in the PCB """
+            """ Exclude components marked as virtual in the PCB """
             self.exclude_smd = False
-            """ KiCad 5: exclude components marked as smd in the PCB """
+            """ Exclude components marked as smd in the PCB """
             self.exclude_tht = False
-            """ KiCad 5: exclude components marked as through-hole in the PCB """
+            """ Exclude components marked as through-hole in the PCB """
+            self.exclude_top = False
+            """ Exclude components on the top side of the PCB """
+            self.exclude_bottom = False
+            """ Exclude components on the bottom side of the PCB  """
         self.add_to_doc('keys', 'Use `dnf_list` for '+str(sorted(DNF)))
         self.add_to_doc('keys', 'Use `dnc_list` for '+str(sorted(DNC)))
 
@@ -119,6 +123,7 @@ class Generic(BaseFilter):  # noqa: F821
         if not self.include_only:  # Nothing to match against, means include all
             return True
         for reg in self.include_only:
+            reg.column = Optionable.solve_field_name(reg.column)
             if reg.skip_if_no_field and not c.is_field(reg.column):
                 # Skip the check if the field doesn't exist
                 continue
@@ -144,6 +149,7 @@ class Generic(BaseFilter):  # noqa: F821
         if not self.exclude_any:  # Nothing to match against, means don't exclude any
             return False
         for reg in self.exclude_any:
+            reg.column = Optionable.solve_field_name(reg.column)
             if reg.skip_if_no_field and not c.is_field(reg.column):
                 # Skip the check if the field doesn't exist
                 continue
@@ -180,6 +186,10 @@ class Generic(BaseFilter):  # noqa: F821
             return exclude
         if self.exclude_tht and comp.tht:
             return exclude
+        if self.exclude_top and not comp.bottom:
+            return exclude
+        if self.exclude_bottom and comp.bottom:
+            return exclude
         # List of references to be excluded
         if self.exclude_refs and (comp.ref in self.exclude_refs or comp.ref_prefix+'*' in self.exclude_refs):
             return exclude
