@@ -14,18 +14,19 @@ Notes about coverage:
 I'm excluding all the Darwin and Windows code from coverage.
 I'm not even sure the values are correct.
 """
-import os
-import re
-import sys
-import json
-from io import StringIO
+import csv
 from glob import glob
-from shutil import copy2
+from io import StringIO
+import json
+import os
 import platform
+import re
+from shutil import copy2
+import sys
 import sysconfig
 from ..gs import GS
 from .. import log
-from ..misc import W_NOCONFIG, W_NOKIENV, W_NOLIBS, W_NODEFSYMLIB, MISSING_WKS
+from ..misc import W_NOCONFIG, W_NOKIENV, W_NOLIBS, W_NODEFSYMLIB, MISSING_WKS, W_MAXDEPTH, W_3DRESVER
 
 # Check python version to determine which version of ConfirParser to import
 if sys.version_info.major >= 3:
@@ -37,6 +38,8 @@ else:  # pragma: no cover (Py2)
 logger = log.get_logger()
 SYM_LIB_TABLE = 'sym-lib-table'
 KICAD_COMMON = 'kicad_common'
+MAXDEPTH = 20
+reported = set()
 
 
 class KiConfError(Exception):
@@ -56,19 +59,48 @@ def un_quote(val):
     return val
 
 
+def parse_len_str(val):
+    if ':' not in val:
+        return val
+    pos = val.index(':')
+    try:
+        c = int(val[:pos])
+    except ValueError:
+        c = None
+        logger.error('Malformed 3D alias entry: '+val)
+    value = val[pos+1:]
+    if c is not None and c != len(value):
+        logger.error('3D alias entry error, expected len {}, but found {}'.format(c, len(value)))
+    return value
+
+
 def expand_env(val, env, extra_env, used_extra=None):
     """ Expand KiCad environment variables """
     if used_extra is None:
         used_extra = [False]
     used_extra[0] = False
-    for var in re.findall(r'\$\{(\S+?)\}', val):
-        if var in env:
-            val = val.replace('${'+var+'}', env[var])
-        elif var in extra_env:
-            val = val.replace('${'+var+'}', extra_env[var])
-            used_extra[0] = True
-        else:
-            logger.error('Unable to expand `{}` in `{}`'.format(var, val))
+    success = replaced = True
+    depth = 0
+    ori_val = val
+    while success and replaced and depth < MAXDEPTH:
+        replaced = False
+        depth += 1
+        if depth == MAXDEPTH:
+            logger.warning(W_MAXDEPTH+'Too much nested variables replacements, possible loop ({})'.format(ori_val))
+            success = False
+        for var in re.findall(r'\$\{(\S+?)\}', val):
+            if var in env:
+                val = val.replace('${'+var+'}', env[var])
+                replaced = True
+            elif var in extra_env:
+                val = val.replace('${'+var+'}', extra_env[var])
+                used_extra[0] = True
+                replaced = True
+            else:
+                success = False
+                if var not in reported:
+                    logger.error('Unable to expand `{}` in `{}`'.format(var, val))
+                    reported.add(var)
     return val
 
 
@@ -113,8 +145,11 @@ class KiConf(object):
     sym_lib_dir = None
     template_dir = None
     footprint_dir = None
+    models_3d_dir = None
+    party_3rd_dir = None
     kicad_env = {}
     lib_aliases = {}
+    aliases_3D = {}
 
     def __init__(self):
         raise AssertionError("KiConf is fully static, no instances allowed")
@@ -128,6 +163,7 @@ class KiConf(object):
         KiConf.kicad_env['KIPRJMOD'] = KiConf.dirname
         KiConf.load_kicad_common()
         KiConf.load_all_lib_aliases()
+        KiConf.load_3d_aliases()
         KiConf.loaded = True
 
     def find_kicad_common():
@@ -136,7 +172,7 @@ class KiConf(object):
         cfg = ''
         if GS.kicad_conf_path:
             cfg = os.path.join(GS.kicad_conf_path, KICAD_COMMON)
-            if GS.ki6():
+            if GS.ki6:
                 cfg += '.json'
             if os.path.isfile(cfg):
                 return cfg
@@ -184,7 +220,7 @@ class KiConf(object):
         return None
 
     def guess_symbol_dir():
-        if GS.ki5():
+        if GS.ki5:
             order = ['library', 'symbols']
         else:
             order = ['symbols', 'library']
@@ -194,7 +230,7 @@ class KiConf(object):
         return guess
 
     def guess_footprint_dir():
-        if GS.ki5():
+        if GS.ki5:
             order = ['modules', 'footprints']
         else:
             order = ['footprints', 'modules']
@@ -208,7 +244,7 @@ class KiConf(object):
 
     def guess_3d_dir():
         modules3d = os.path.join('modules', 'packages3d')
-        if GS.ki5():
+        if GS.ki5:
             order = [modules3d, '3dmodels']
         else:
             order = ['3dmodels', modules3d]
@@ -223,7 +259,7 @@ class KiConf(object):
             home = os.environ.get('HOME')
             if home is None:
                 return None
-            if GS.ki6():
+            if GS.ki6:
                 name = os.path.join(home, '.local', 'share', 'kicad', '6.0', 'template')
                 if os.path.isdir(name):
                     return name
@@ -299,7 +335,7 @@ class KiConf(object):
         if not no_dir:
             base_name += '_DIR'
         names = []
-        if GS.ki6() and ki6_diff:
+        if GS.ki6 and ki6_diff:
             # KiCad 6 specific name goes first when using KiCad 6
             names.append('KICAD6_'+base_name)
         # KiCad 5 names, allowed even when using KiCad 6
@@ -326,13 +362,22 @@ class KiConf(object):
                 logger.debug('Using {}="{}" (from KiCad config)'.format(n, val))
             if val is not None and ret_val is None:
                 ret_val = val
+        # Make sure all the versions point to the same place
+        # It helps if we are using KiCad 6 but have the KiCad 5 names defined,
+        # KiCad will use them, but any mention to the KiCad 6 version won't be
+        # valid for KiBot unless we explicitly define it.
+        if ret_val is not None:
+            for n in names:
+                if n not in os.environ or n not in KiConf.kicad_env:
+                    os.environ[n] = ret_val
+                    KiConf.kicad_env[n] = ret_val
         return ret_val
 
     def _set_env_var(base_name, val, ki6_diff, no_dir):
         """ Sets the environment and the internal list """
         if not no_dir:
             base_name += '_DIR'
-        if GS.ki6() and ki6_diff:
+        if GS.ki6 and ki6_diff:
             name = 'KICAD6_'+base_name
         else:
             name = 'KICAD_'+base_name
@@ -341,7 +386,7 @@ class KiConf(object):
         logger.debug('Using {}="{}" (guessed)'.format(name, val))
 
     def _solve_var(name, member, desc, guesser, old=None, only_old=False, ki6_diff=True, only_k6=False, no_dir=False):
-        if only_k6 and GS.ki5():
+        if only_k6 and GS.ki5:
             return
         val = KiConf._look_env_var(name, old, only_old, ki6_diff, no_dir)
         if val is not None:
@@ -356,6 +401,7 @@ class KiConf(object):
                     os.environ[old] = val
             else:
                 logger.warning(W_NOLIBS + 'Unable to find KiCad '+desc)
+        return val
 
     def load_kicad_common():
         # Try to figure out KiCad configuration file
@@ -364,7 +410,7 @@ class KiConf(object):
             # Get the environment variables
             logger.debug('Reading KiCad config from `{}`'.format(cfg))
             KiConf.config_dir = os.path.dirname(cfg)
-            if GS.ki5():
+            if GS.ki5:
                 # All environment vars should be here
                 KiConf.load_ki5_env(cfg)
             else:
@@ -435,6 +481,30 @@ class KiConf(object):
         # Load the project's table
         KiConf.load_lib_aliases(os.path.join(KiConf.dirname, SYM_LIB_TABLE))
 
+    def load_3d_aliases():
+        if not KiConf.config_dir:
+            return
+        fname = os.path.join(KiConf.config_dir, '3d', '3Dresolver.cfg')
+        if not os.path.isfile(fname):
+            logger.debug('No 3D aliases ({})'.format(fname))
+            return
+        logger.debug('Loading 3D aliases from '+fname)
+        with open(fname) as f:
+            reader = csv.reader(f)
+            head = next(reader)
+            if len(head) != 1 or head[0] != '#V1':
+                logger.warning(W_3DRESVER, 'Unsupported 3D resolver version ({})'.format(head))
+            for r in reader:
+                if len(r) != 3:
+                    logger.error("3D resolver doesn't contain three values ({})".format(r))
+                    continue
+                name = parse_len_str(r[0])
+                value = parse_len_str(r[1])
+                # Discard the comment (2)
+                logger.debugl(1, '- {}={}'.format(name, value))
+                KiConf.aliases_3D[name] = value
+        logger.debugl(1, 'Finished loading 3D aliases')
+
     def fix_page_layout_k6_key(key, data, dest_dir):
         if key in data:
             section = data[key]
@@ -512,13 +582,17 @@ class KiConf(object):
         if not project:
             return None, None
         KiConf.init(GS.pcb_file)
-        if GS.ki5():
+        if GS.ki5:
             return KiConf.fix_page_layout_k5(project, dry)
         return KiConf.fix_page_layout_k6(project, dry)
 
-    def expand_env(name, used_extra=None):
+    def expand_env(name, used_extra=None, ref_dir=None):
         if used_extra is None:
             used_extra = [False]
         if not name:
             return name
-        return os.path.abspath(expand_env(un_quote(name), KiConf.kicad_env, GS.load_pro_variables(), used_extra))
+        expanded = expand_env(un_quote(name), KiConf.kicad_env, GS.load_pro_variables(), used_extra)
+        # Don't try to get the absolute path for something that starts with a variable that we couldn't expand
+        if ref_dir is None:
+            ref_dir = os.getcwd()
+        return expanded if expanded.startswith('${') else os.path.normpath(os.path.join(ref_dir, expanded))
