@@ -2,11 +2,14 @@
 """Expander core; essentially, how to apply a macro invocation."""
 
 __all__ = ["MacroExpansionError", "MacroExpanderMarker", "Done",
-           "BaseMacroExpander", "global_postprocess"]
+           "BaseMacroExpander", "global_postprocess",
+           "add_postprocessor", "remove_postprocessor"]
 
-from ast import NodeTransformer, AST
-from contextlib import contextmanager
+from ast import AST, NodeTransformer
 from collections import ChainMap
+from contextlib import contextmanager
+from copy import deepcopy
+from typing import Any, Callable, Dict, List
 
 from .astfixers import fix_ctx, fix_locations
 from .markers import ASTMarker, delete_markers
@@ -14,7 +17,14 @@ from .utils import flatten, format_location
 
 # Global macro bindings shared across all expanders in the current process.
 # This is used by `mcpyrate.quotes` for hygienically captured macro functions.
-global_bindings = {}
+#
+# TODO: In reality, a macro function is not just any callable, but it takes certain
+# (especially named) arguments, and its return type is `Union[AST, List[AST], None]`,
+# but as of 3.1.0, we skim over this detail.
+global_bindings: Dict[str, Callable[..., Any]] = {}
+
+# User function hook for `global_postprocess`.
+global_postprocessors: List[Callable[..., Any]] = []
 
 class MacroExpansionError(Exception):
     """Base class for errors specific to macro expansion.
@@ -83,11 +93,12 @@ class BaseMacroExpander(NodeTransformer):
         filename: full path to `.py` file being expanded, for error reporting
     """
 
-    def __init__(self, bindings, filename):
+    def __init__(self, bindings: Dict[str, Callable[..., Any]], filename: str):
         self.local_bindings = bindings
         self.bindings = ChainMap(self.local_bindings, global_bindings)
         self.filename = filename
         self.recursive = True
+        self._debughook = None  # see `mcpyrate.debug.step_expansion`
 
     def visit(self, tree):
         """Expand macros in `tree`, using current setting for recursive mode.
@@ -137,7 +148,7 @@ class BaseMacroExpander(NodeTransformer):
         with self._recursive_mode(False):
             return Done(self.visit(tree))
 
-    def _recursive_mode(self, isrecursive):
+    def _recursive_mode(self, isrecursive: bool):
         """Context manager. Change recursive mode, restoring the old mode when the context exits."""
         @contextmanager
         def recursive_mode():
@@ -149,6 +160,30 @@ class BaseMacroExpander(NodeTransformer):
                 self.recursive = wasrecursive
         return recursive_mode()
 
+    def debughook(self, hook: Callable[..., None]):
+        """Context manager. Temporarily set a debug hook, restoring the old one when the context exits.
+
+        The debug hook, if one is installed, is called whenever a macro expands.
+
+        The hook receives the following arguments, passed positionally in this order:
+            invocationsubtreeid: int, the `id()` of the *original* macro invocation subtree
+                                 before anything was done to it.
+            invocationtree:      AST, (a deepcopy of) the macro invocation subtree before expansion.
+            expandedtree:        AST, the replacement subtree after expanding once. It's the actual live copy.
+            macroname:           str, name of the macro that was applied, as it appears in this expander's
+                                 bindings.
+            macrofunction:       callable, the macro function that was applied.
+        """
+        @contextmanager
+        def debughook_context():
+            oldhook = self._debughook
+            try:
+                self._debughook = hook
+                yield
+            finally:
+                self._debughook = oldhook
+        return debughook_context()
+
     def expand(self, syntax, target, macroname, tree, sourcecode, kw=None):
         """Expand a macro invocation.
 
@@ -180,7 +215,21 @@ class BaseMacroExpander(NodeTransformer):
 
                           Very rarely needed; if you need it, you'll know.
 
-                          **CAUTION**: not a copy, or at most a shallow copy.
+                          **CAUTION**: Not a copy, or at most a shallow copy.
+
+                          **CAUTION**: The caller, i.e. the concrete expander's
+                                       visit method that corresponds to the macro
+                                       invocation `syntax` being processed,
+                                       is responsible for providing a complete,
+                                       unmodified `invocation`.
+
+                                       So for example, if your concrete expander
+                                       needs to pop a block macro, be sure to stash
+                                       an appropriately prepared copy of the invocation
+                                       (so that it looks like the original one)
+                                       before you edit the original. This is used for
+                                       debug purposes, so preserving the appearance
+                                       of the original invocation is important.
 
         To send additional named arguments from the actual expander to the
         macro function, place them in a dictionary and pass that dictionary
@@ -197,10 +246,10 @@ class BaseMacroExpander(NodeTransformer):
             # Resolve macro binding.
             macro = self.isbound(macroname)
             if not macro:  # pragma: no cover
-                raise MacroApplicationError(f"{loc}\nin {syntax} macro invocation for '{macroname}': the name '{macroname}' is not bound to a macro.")
+                raise MacroApplicationError(f"{loc}\n""in {syntax} macro invocation for '{macroname}': the name '{macroname}' is not bound to a macro.")
 
             # Expand the macro.
-            expansion = _apply_macro(macro, tree, kw)
+            expansion = self._apply_macro(macro, tree, kw, macroname, target)
 
             # Convert possible iterable result to `list`, then typecheck macro output.
             try:
@@ -223,7 +272,7 @@ class BaseMacroExpander(NodeTransformer):
 
         # If something went wrong, generate a standardized macro use site report.
         except Exception as err:
-            msg = f"{loc}\nin {syntax} macro invocation for '{macroname}'"
+            msg = f"{loc}\n""in {syntax} macro invocation for '{macroname}'"
             if isinstance(err, MacroApplicationError) and err.__cause__:
                 # Telescope nested use site reports, by keeping the original
                 # traceback and `__cause__`, but combining the messages.
@@ -254,6 +303,32 @@ class BaseMacroExpander(NodeTransformer):
 
         return self._visit_expansion(expansion, target)
 
+    def _apply_macro(self, macro, tree, kw, macroname, target):
+        """Execute `macro` on `tree`, with the dictionary `kw` unpacked into macro's named arguments.
+
+        `macro` is the macro function.
+
+        `tree` is the AST to apply the macro to. This is what the macro gets as its `tree` argument.
+
+        `kw` is a dictionary-like, unpacked into the macro's named arguments.
+
+        `macroname` is the name (str) the macro function `macro` is bound as in this expander.
+
+        `target` is the whole macro invocation AST. A deepcopy is passed to the debug hook (if installed).
+        """
+        # There's no guarantee whether `macro` edits the AST in-place or produces a new tree,
+        # so to produce reliable debug information, we should take the old `id` and a deep copy
+        # of the macro invocation.
+        if self._debughook:
+            oldid = id(target)
+            oldtree = deepcopy(target)  # including the macro invocation itself
+
+        newtree = macro(tree, **kw)
+
+        if self._debughook:
+            self._debughook(oldid, oldtree, newtree, macroname, macro)
+        return newtree
+
     def _visit_expansion(self, expansion, target):
         """Perform local postprocessing.
 
@@ -278,10 +353,6 @@ class BaseMacroExpander(NodeTransformer):
             return bindings[name]
         return False
 
-def _apply_macro(macro, tree, kw):
-    """Execute `macro` on `tree`, with the dictionary `kw` unpacked into macro's named arguments."""
-    return macro(tree, **kw)
-
 
 # Final postprocessing for the top-level walk can't be done at the end of the
 # entry points `visit_once` and `visit_recursively`, because it is valid for a
@@ -296,8 +367,53 @@ def global_postprocess(tree):
     to Python's `compile`.
     """
     tree = delete_markers(tree, cls=MacroExpanderMarker)
+
     # A name macro, appearing as an assignment target, gets the wrong ctx,
     # because when expanding the name macro, the expander sees only the Name
     # node, and thus puts an `ast.Load` there as the ctx.
     tree = fix_ctx(tree, copy_seen_nodes=True)
+
+    # Run the user hooks, if any.
+    for custom_postprocessor in global_postprocessors:
+        tree = custom_postprocessor(tree)
+
     return tree
+
+def add_postprocessor(function):
+    """Add a custom postprocessor for the top-level expansion.
+
+    `function` must accept `AST` and `list` of `AST`, and return
+    the same type.
+
+    Custom postprocessors are automatically called by `global_postprocess`
+    (which is called just after macro expansion for a module ends), in the
+    order they were registered by calling `add_postprocessor`.
+
+    This e.g. allows a macro library to use its own `ASTMarker` subclasses
+    for internal communication between macros, and delete (only) its own
+    markers when done.
+
+    Put something like this somewhere in your library initialization code
+    (preferably somewhere it's easy to discover for someone reading your
+    code, such as in a package `__init__.py`):
+
+        from functools import partial
+
+        import mcpyrate.core
+        from mcpyrate.markers import delete_markers
+
+        _delete_my_markers = partial(delete_markers,
+                                     cls=MyCustomMarkerBase)
+        mcpyrate.core.add_postprocessor(_delete_my_markers)
+    """
+    if function not in global_postprocessors:
+        global_postprocessors.append(function)
+
+def remove_postprocessor(function):
+    """Remove a previously added custom postprocessor.
+
+    The `function` is the callable object that was registered as a
+    postprocessor using `add_postprocessor`.
+    """
+    if function in global_postprocessors:
+        global_postprocessors.remove(function)
