@@ -1,19 +1,25 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020-2022 Salvador E. Tropea
-# Copyright (c) 2020-2022 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2023 Salvador E. Tropea
+# Copyright (c) 2020-2023 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 """ Base class for output options """
+import difflib
+import inspect
 import os
 import re
-import inspect
 from re import compile
 from .error import KiPlotConfigurationError
 from .gs import GS
-from .misc import W_UNKOPS
+from .misc import W_UNKOPS, DISTRIBUTORS_STUBS, DISTRIBUTORS_STUBS_SEPS
 from . import log
 
 logger = log.get_logger()
+HEX_DIGIT = '[A-Fa-f0-9]{2}'
+INVALID_CHARS = r'[?%*:|"<>]'
+PATTERNS_DEP = ['%c', '%d', '%F', '%f', '%p', '%r']
+for n in range(1, 10):
+    PATTERNS_DEP.append('%C'+str(n))
 
 
 def do_filter(v):
@@ -31,8 +37,10 @@ class Optionable(object):
     _str_values_re = compile(r"string=.*\] \[([^\]]+)\]")
     _num_range_re = compile(r"number=.*\] \[(-?\d+),(-?\d+)\]")
     _default = None
-    _color_re = re.compile(r"#[A-Fa-f0-9]{6}$")
-    _color_re_a = re.compile(r"#[A-Fa-f0-9]{8}$")
+
+    _color_re = re.compile(r"#("+HEX_DIGIT+"){3}$")
+    _color_re_a = re.compile(r"#("+HEX_DIGIT+"){4}$")
+    _color_re_component = re.compile(HEX_DIGIT)
 
     def __init__(self):
         self._unkown_is_error = False
@@ -42,6 +50,10 @@ class Optionable(object):
         # File/directory pattern expansion
         self._expand_id = ''
         self._expand_ext = ''
+        # Used to indicate we have an output pattern and it must be suitable to generate multiple files
+        self._output_multiple_files = False
+        # The KiBoM output uses "variant" for the KiBoM variant, not for KiBot variants
+        self._variant_is_real = True
         super().__init__()
         for var in ['output', 'variant', 'units', 'hide_excluded']:
             glb = getattr(GS, 'global_'+var)
@@ -121,13 +133,19 @@ class Optionable(object):
     def _perform_config_mapping(self):
         """ Map the options to class attributes """
         attrs = self.get_attrs_for()
+        if not isinstance(self._tree, dict):
+            raise KiPlotConfigurationError('Found {} instead of dict'.format(type(self._tree)))
         for k, v in self._tree.items():
             # Map known attributes and avoid mapping private ones
             if (k[0] == '_') or (k not in attrs):
                 if self._unkown_is_error:
                     valid = list(filter(lambda x: x[0] != '_', attrs.keys()))
-                    raise KiPlotConfigurationError("Unknown {}option `{}`. Valid options: {}".
-                                                   format(self._error_context, k, valid))
+                    msg = "Unknown {}option `{}`.".format(self._error_context, k)
+                    possible = difflib.get_close_matches(k, valid, n=1)
+                    if possible:
+                        msg += " Did you meen {}?".format(possible[0])
+                    msg += " Valid options: {}".format(valid)
+                    raise KiPlotConfigurationError(msg)
                 logger.warning(W_UNKOPS + "Unknown {}option `{}`".format(self._error_context, k))
                 continue
             # Check the data type
@@ -192,7 +210,18 @@ class Optionable(object):
                                 new_val.append(element)
                         v = new_val
             # Seems to be ok, map it
-            setattr(self, alias if is_alias else k, v)
+            dest_name = alias if is_alias else k
+            setattr(self, dest_name, v)
+            self.set_user_defined(dest_name)
+
+    def set_user_defined(self, name):
+        setattr(self, '_{}_user_defined'.format(name), True)
+
+    def get_user_defined(self, name):
+        name = '_{}_user_defined'.format(name)
+        if hasattr(self, name):
+            return getattr(self, name)
+        return False
 
     def set_tree(self, tree):
         self._tree = tree
@@ -202,6 +231,8 @@ class Optionable(object):
         if self._tree and not self._configured:
             self._perform_config_mapping()
             self._configured = True
+        if self._output_multiple_files and ('%i' not in self.output or '%x' not in self.output):
+            raise KiPlotConfigurationError('The output pattern must contain %i and %x, otherwise file names will collide')
 
     def get_attrs_for(self):
         """ Returns all attributes """
@@ -222,7 +253,10 @@ class Optionable(object):
         """ Returns the text to add for the current variant.
             Also try with the globally defined variant.
             If no variant is defined an empty string is returned. """
-        if hasattr(self, 'variant') and self.variant and hasattr(self.variant, 'file_id'):
+        if not self._variant_is_real:
+            return self.variant
+        if hasattr(self, 'variant') and self.variant:
+            self.variant = GS.solve_variant(self.variant)
             return self.variant.file_id
         return Optionable._find_global_variant()
 
@@ -236,10 +270,31 @@ class Optionable(object):
         """ Returns the name for the current variant.
             Also try with the globally defined variant.
             If no variant is defined an empty string is returned. """
-        if hasattr(self, 'variant') and self.variant and hasattr(self.variant, 'name'):
+        if not self._variant_is_real:
+            return self.variant
+        if hasattr(self, 'variant') and self.variant:
+            self.variant = GS.solve_variant(self.variant)
             return self.variant.name
         return Optionable._find_global_variant_name()
 
+    @staticmethod
+    def _find_global_subpcb():
+        if GS.solved_global_variant and GS.solved_global_variant._sub_pcb:
+            return GS.solved_global_variant._sub_pcb.name
+        return ''
+
+    def _find_subpcb(self):
+        """ Returns the name of the sub-PCB.
+            Also try with the globally defined variant.
+            If no variant is defined an empty string is returned. """
+        if not self._variant_is_real:
+            return ''
+        if hasattr(self, 'variant') and self.variant:
+            self.variant = GS.solve_variant(self.variant)
+            if self.variant._sub_pcb:
+                return self.variant._sub_pcb.name
+        return Optionable._find_global_subpcb()
+
     def expand_filename_common(self, name, parent):
         """ Expansions common to the PCB and Schematic """
         # PCB expansions, explicit
@@ -271,6 +326,7 @@ class Optionable(object):
             name = name.replace('%i', self._expand_id)
             name = name.replace('%v', _cl(self._find_variant()))
             name = name.replace('%V', _cl(self._find_variant_name()))
+            name = name.replace('%S', _cl(self._find_subpcb()))
             name = name.replace('%x', self._expand_ext)
             replace_id = ''
             if parent and hasattr(parent, 'output_id'):
@@ -293,7 +349,7 @@ class Optionable(object):
         # Replace KiCad 6 variables first
         name = GS.expand_text_variables(name)
         # Determine if we need to expand SCH and/or PCB related data
-        has_dep_exp = any(map(lambda x: x in name, ['%c', '%d', '%F', '%f', '%p', '%r', '%C1', '%C2', '%C3', '%C4']))
+        has_dep_exp = any(map(lambda x: x in name, PATTERNS_DEP))
         do_sch = is_sch and has_dep_exp
         # logger.error(name + '  is_sch ' +str(is_sch)+"   "+ str(do_sch))
         # raise
@@ -331,8 +387,16 @@ class Optionable(object):
         name = GS.expand_text_variables(name)
         if make_safe:
             # sanitize the name to avoid characters illegal in file systems
-            name = name.replace('\\', '/')
-            name = re.sub(r'[?%*:|"<>]', '_', name)
+            if GS.on_windows:
+                # Here \ *is* valid
+                if len(name) >= 2 and name[0].isalpha() and name[1] == ':':
+                    # This name starts with a drive letter, : is valid in the first 2
+                    name = name[:2]+re.sub(INVALID_CHARS, '_', name[2:])
+                else:
+                    name = re.sub(INVALID_CHARS, '_', name)
+            else:
+                name = name.replace('\\', '/')
+                name = re.sub(INVALID_CHARS, '_', name)
         if GS.debug_level > 3:
             logger.debug('Expanded `{}`'.format(name))
         return name
@@ -350,7 +414,7 @@ class Optionable(object):
         return Optionable.expand_filename_both(self, name)
 
     @staticmethod
-    def force_list(val, comma_sep=True):
+    def force_list(val, comma_sep=True, lower_case=False):
         """ Used for values that accept a string or a list of strings.
             The string can be a comma separated list """
         if isinstance(val, type):
@@ -366,21 +430,106 @@ class Optionable(object):
             else:
                 # Empty string
                 val = []
+        if lower_case:
+            return [c.lower() for c in val]
         return val
 
     @classmethod
     def get_default(cls):
         return cls._default
 
+    def validate_color_str(self, color):
+        return self._color_re.match(color) or self._color_re_a.match(color)
+
     def validate_color(self, name):
-        color = getattr(self, name)
-        if not self._color_re.match(color) and not self._color_re_a.match(color):
+        if not self.validate_color_str(getattr(self, name)):
             raise KiPlotConfigurationError('Invalid color for `{}` use `#rrggbb` or `#rrggbbaa` with hex digits'.format(name))
 
     def validate_colors(self, names):
         for color in names:
             self.validate_color(color)
 
+    def parse_one_color(self, color):
+        res = self._color_re_component.findall(color)
+        alpha = 1.0
+        if len(res) > 3:
+            alpha = int(res[3], 16)/255.0
+        return (int(res[0], 16)/255.0, int(res[1], 16)/255.0, int(res[2], 16)/255.0, alpha)
+
+    def color_to_rgb(self, color):
+        index = 4 if len(color) > 4 else 0
+        alpha = color[index+3]
+        if alpha == 1.0:
+            return "rgb({}, {}, {})".format(round(color[index]*255.0), round(color[index+1]*255.0),
+                                            round(color[index+2]*255.0))
+        return "rgba({}, {}, {}, {})".format(round(color[index]*255.0), round(color[index+1]*255.0),
+                                             round(color[index+2]*255.0), alpha)
+
+    def color_str_to_rgb(self, color):
+        return self.color_to_rgb(self.parse_one_color(color))
+
+    @staticmethod
+    def _solve_field_name(field, empty_when_none):
+        """ Applies special replacements for field """
+        # The global name for the LCSC part field
+        if GS.global_field_lcsc_part:
+            logger.debug('- User selected: '+GS.global_field_lcsc_part)
+            return GS.global_field_lcsc_part
+        # No name defined, try to figure out
+        if not GS.sch and GS.sch_file:
+            GS.load_sch()
+        if not GS.sch:
+            logger.debug("- No schematic loaded, can't search the name")
+            return '' if empty_when_none else 'LCSC#'
+        if hasattr(GS.sch, '_field_lcsc_part'):
+            return GS.sch._field_lcsc_part
+        # Look for a common name
+        fields = {f.lower() for f in GS.sch.get_field_names([])}
+        for stub in DISTRIBUTORS_STUBS:
+            fld = 'lcsc'+stub
+            if fld in fields:
+                GS.sch._field_lcsc_part = fld
+                return fld
+            if stub != '#':
+                for sep in DISTRIBUTORS_STUBS_SEPS:
+                    fld = 'lcsc'+sep+stub
+                    if fld in fields:
+                        GS.sch._field_lcsc_part = fld
+                        return fld
+        if 'lcsc' in fields:
+            GS.sch._field_lcsc_part = 'LCSC'
+            return 'LCSC'
+        # Look for a field that only contains LCSC codes
+        comps = GS.sch.get_components()
+        lcsc_re = re.compile(r'C\d+')
+        for f in fields:
+            found = False
+            for c in comps:
+                val = c.get_field_value(f).strip()
+                if not val:
+                    continue
+                if lcsc_re.match(val):
+                    found = True
+                else:
+                    found = False
+                    break
+            if found:
+                GS.sch._field_lcsc_part = f
+                return f
+        logger.debug('- No LCSC field found')
+        res = '' if empty_when_none else 'LCSC#'
+        GS.sch._field_lcsc_part = res
+        return res
+
+    @staticmethod
+    def solve_field_name(field, empty_when_none=False):
+        if field != '_field_lcsc_part':
+            return field
+        logger.debug('Looking for LCSC field name')
+        field = Optionable._solve_field_name(field, empty_when_none)
+        logger.debug('Using {} as LCSC field name'.format(field))
+        return field
+
 
 class BaseOptions(Optionable):
     """ A class to validate and hold output options.
@@ -396,6 +545,10 @@ class BaseOptions(Optionable):
         """ Looks for a mandatory dependency """
         return GS.check_tool_dep(self._parent.type, name, fatal=True)
 
+    def ensure_tool_get_ver(self, name):
+        """ Looks for a mandatory dependency, also returns its version """
+        return GS.check_tool_dep_get_ver(self._parent.type, name, fatal=True)
+
     def check_tool(self, name):
         """ Looks for a dependency """
         return GS.check_tool_dep(self._parent.type, name, fatal=False)
@@ -410,3 +563,56 @@ class BaseOptions(Optionable):
         self._expand_id = cur_id
         self._expand_ext = cur_ext
         return res
+
+
+class PanelOptions(BaseOptions):
+    """ A class for options that uses KiKit's units """
+    _num_regex = re.compile(r'([\d\.]+)(mm|cm|dm|m|mil|inch|in)')
+    _ang_regex = re.compile(r'([\d\.]+)(deg|°|rad)')
+
+    def add_units(self, ops, def_units=None, convert=False):
+        if def_units is None:
+            def_units = self._parent._parent.units
+        for op in ops:
+            val = getattr(self, op)
+            _op = '_'+op
+            if val is None:
+                if convert:
+                    setattr(self, _op, 0)
+                continue
+            if isinstance(val, (int, float)):
+                setattr(self, op, str(val)+def_units)
+                if convert:
+                    setattr(self, _op, val*GS.kikit_units_to_kicad[def_units])
+            else:
+                m = PanelOptions._num_regex.match(val)
+                if m is None:
+                    raise KiPlotConfigurationError('Malformed value `{}: {}` must be a number and units'.format(op, val))
+                num = m.group(1)
+                try:
+                    num_d = float(num)
+                except ValueError:
+                    num_d = None
+                if num_d is None:
+                    raise KiPlotConfigurationError('Malformed number in `{}` ({})'.format(op, num))
+                if convert:
+                    setattr(self, _op, num_d*GS.kikit_units_to_kicad[m.group(2)])
+
+    def add_angle(self, ops, def_units=None):
+        if def_units is None:
+            def_units = self._parent._parent.units
+        for op in ops:
+            val = getattr(self, op)
+            if isinstance(val, (int, float)):
+                setattr(self, op, str(val)+def_units)
+            else:
+                m = PanelOptions._ang_regex.match(val)
+                if m is None:
+                    raise KiPlotConfigurationError('Malformed angle `{}: {}` must be a number and its type'.format(op, val))
+                num = m.group(1)
+                try:
+                    num_d = float(num)
+                except ValueError:
+                    num_d = None
+                if num_d is None:
+                    raise KiPlotConfigurationError('Malformed number in `{}` ({})'.format(op, num))
