@@ -11,7 +11,7 @@ from subprocess import check_output, STDOUT, CalledProcessError
 from .gs import GS
 from .error import KiPlotConfigurationError
 from .kiplot import config_output, get_output_dir, run_output
-from .misc import MISSING_TOOL, WRONG_INSTALL, WRONG_ARGUMENTS, INTERNAL_ERROR, W_NOTPDF
+from .misc import MISSING_TOOL, WRONG_INSTALL, WRONG_ARGUMENTS, INTERNAL_ERROR, W_NOTPDF, MISSING_FILES, W_NOMATCH
 from .optionable import Optionable, BaseOptions
 from .registrable import RegOutput
 from .create_pdf import create_pdf_from_pages
@@ -26,13 +26,13 @@ class FilesList(Optionable):
         super().__init__()
         with document:
             self.source = '*.pdf'
-            """ File names to add, wildcards allowed. Use ** for recursive match.
+            """ *File names to add, wildcards allowed. Use ** for recursive match.
                 By default this pattern is applied to the output dir specified with `-d` command line option.
                 See the `from_cwd` option """
             self.from_cwd = False
             """ Use the current working directory instead of the dir specified by `-d` """
             self.from_output = ''
-            """ Collect files from the selected output.
+            """ *Collect files from the selected output.
                 When used the `source` option is ignored """
             self.filter = r'.*\.pdf'
             """ A regular expression that source files must match """
@@ -42,9 +42,9 @@ class PDFUniteOptions(BaseOptions):
     def __init__(self):
         with document:
             self.output = GS.def_global_output
-            """ Name for the generated PDF (%i=name of the output %x=pdf) """
+            """ *Name for the generated PDF (%i=name of the output %x=pdf) """
             self.outputs = FilesList
-            """ [list(dict)] Which files will be included """
+            """ *[list(dict)] Which files will be included """
             self.use_external_command = False
             """ Use the `pdfunite` tool instead of PyPDF2 Python module """
         super().__init__()
@@ -88,12 +88,15 @@ class PDFUniteOptions(BaseOptions):
                 source = f.expand_filename_both(f.source, make_safe=False)
                 files_list = glob.iglob(os.path.join(out_dir, source), recursive=True)
             # Filter and adapt them
+            old_len = len(files)
             for fname in filter(re.compile(f.filter).match, files_list):
                 fname_real = os.path.realpath(fname)
                 # Avoid including the output
                 if fname_real == output_real:
                     continue
                 files.append(fname_real)
+            if len(files) == old_len:
+                logger.warning(W_NOMATCH+'No match found for `{}`'.format(f.from_output if f.from_output else f.source))
         return files
 
     def get_targets(self, out_dir):
@@ -128,6 +131,9 @@ class PDFUniteOptions(BaseOptions):
                 sig = f.read(4)
                 if sig != b'%PDF':
                     logger.warning(W_NOTPDF+'Joining a non PDF file `{}`, will most probably fail'.format(fn))
+        if len(files) < 2:
+            logger.error('At least two files must be joined ({})'.format(files))
+            exit(MISSING_FILES)
         logger.debug('Generating `{}` PDF'.format(output))
         if os.path.isfile(output):
             os.remove(output)
@@ -146,7 +152,7 @@ class PDFUnite(BaseOutput):  # noqa: F821
         super().__init__()
         with document:
             self.options = PDFUniteOptions
-            """ [dict] Options for the `pdfunite` output """
+            """ *[dict] Options for the `pdfunite` output """
         self._none_related = True
 
     def get_dependencies(self):
