@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2022 Salvador E. Tropea
-# Copyright (c) 2022 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2022-2023 Salvador E. Tropea
+# Copyright (c) 2022-2023 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 """
@@ -11,7 +11,7 @@ Dependencies:
     url_down: https://github.com/jgm/pandoc/releases
     debian: pandoc
     arch: pandoc
-    extra_deb: ['texlive-latex-base', 'texlive-latex-recommended']
+    extra_deb: ['texlive', 'texlive-latex-base', 'texlive-latex-recommended']
     extra_arch: ['texlive-core']
     comments: 'In CI/CD environments: the `kicad_auto_test` docker image contains it.'
 """
@@ -31,6 +31,7 @@ from .kiplot import config_output
 from .dep_downloader import get_dep_data
 from .macros import macros, document, output_class  # noqa: F401
 from . import log
+from . import __version__
 
 logger = log.get_logger()
 INF = float('inf')
@@ -214,6 +215,7 @@ class ReportOptions(BaseOptions):
         dep = get_dep_data('report', 'PanDoc')
         deb_text = 'In Debian/Ubuntu environments: install '+list_nice([dep.deb_package]+dep.extra_deb)
         self._help_do_convert += ".\n"+'\n'.join(dep.comments)+'\n'+deb_text
+        self._shown_defined = False
 
     def config(self, parent):
         super().config(parent)
@@ -222,7 +224,7 @@ class ReportOptions(BaseOptions):
             self.template = self.template[:-6]
             self.to_ascii = True
         if self.template.lower() in ('full', 'simple', 'full_svg'):
-            self.template = os.path.abspath(os.path.join(os.path.dirname(__file__), 'report_templates',
+            self.template = os.path.abspath(os.path.join(GS.get_resource_path('report_templates'),
                                             'report_'+self.template.lower()+'.txt'))
         if not os.path.isabs(self.template):
             self.template = os.path.expandvars(os.path.expanduser(self.template))
@@ -284,7 +286,10 @@ class ReportOptions(BaseOptions):
                     rep = str(val)
                 line = line.replace('${'+var_ori+'}', rep)
             else:
-                print('Error: Unable to expand `{}`'.format(var))
+                logger.error('Unable to expand `{}`'.format(var))
+                if not self._shown_defined:
+                    self._shown_defined = True
+                    logger.error('Defined values: {}'.format([v for v in defined.keys() if v[0] != '_']))
         return line
 
     def context_defined_tracks(self, line):
@@ -445,11 +450,13 @@ class ReportOptions(BaseOptions):
         draw_type = 'DRAWSEGMENT' if GS.ki5 else 'PCB_SHAPE'
         for d in board.GetDrawings():
             if d.GetClass() == draw_type and d.GetLayer() == edge_layer:
+                bb = GS.get_shape_bbox(d)
+                start = bb.GetOrigin()
+                end = bb.GetEnd()
                 if x1 is None:
-                    p = d.GetStart()
-                    x1 = x2 = p.x
-                    y1 = y2 = p.y
-                for p in [d.GetStart(), d.GetEnd()]:
+                    x1 = x2 = start.x
+                    y1 = y2 = start.y
+                for p in [start, end]:
                     x2 = max(x2, p.x)
                     y2 = max(y2, p.y)
                     x1 = min(x1, p.x)
@@ -458,11 +465,13 @@ class ReportOptions(BaseOptions):
         for m in GS.get_modules():
             for gi in m.GraphicalItems():
                 if gi.GetClass() == 'MGRAPHIC' and gi.GetLayer() == edge_layer:
+                    bb = GS.get_shape_bbox(gi)
+                    start = bb.GetOrigin()
+                    end = bb.GetEnd()
                     if x1 is None:
-                        p = gi.GetStart()
-                        x1 = x2 = p.x
-                        y1 = y2 = p.y
-                    for p in [gi.GetStart(), gi.GetEnd()]:
+                        x1 = x2 = start.x
+                        y1 = y2 = start.y
+                    for p in [start, end]:
                         x2 = max(x2, p.x)
                         y2 = max(y2, p.y)
                         x1 = min(x1, p.x)
@@ -487,6 +496,32 @@ class ReportOptions(BaseOptions):
             hole_ec = hole
         return oar, oar_ec, hole_ec
 
+    def analyze_oar(self, oar_t, oar_ec_t, is_pth, min_oar, pad, dr_x_real, dr_y_real, pad_sz, dr):
+        """ Check the computed OAR and choose if we use it or not.
+            Inform anomalies to the user. """
+        if oar_t > 0:
+            if is_pth:
+                # For plated holes we always use it and report anomalies
+                self.oar_pads = min(self.oar_pads, oar_t)
+                self.oar_pads_ec = min(self.oar_pads_ec, oar_ec_t)
+                if oar_t < min_oar:
+                    logger.warning(W_WRONGOAR+"Really small OAR detected ({} mm) for pad {} using drill tool ({}, {})".
+                                   format(to_mm(oar_t, 4), get_pad_info(pad), to_mm(dr_x_real), to_mm(dr_y_real)))
+                    if pad_sz == dr:
+                        logger.warning(W_WRONGOAR+"Try adjusting the drill size to an available drill tool")
+            else:
+                # For non plated holes KiCad doesn't even create a pad if pad_sz == dr
+                if pad_sz != dr:
+                    self.oar_pads = min(self.oar_pads, oar_t)
+                    self.oar_pads_ec = min(self.oar_pads_ec, oar_ec_t)
+        elif oar_t < 0:
+            # The negative value can be a result of converting the drill size to a real drill size
+            # So we inform it only if the pad and drill are different
+            if pad_sz != dr and is_pth:
+                logger.warning(W_WRONGOAR+"Negative OAR detected for pad "+get_pad_info(pad))
+        elif oar_t == 0 and is_pth:
+            logger.warning(W_WRONGOAR+"Plated pad without copper "+get_pad_info(pad))
+
     def collect_data(self, board):
         ds = board.GetDesignSettings()
         self.extra_pth_drill = GS.global_extra_pth_drill*pcbnew.IU_PER_MM
@@ -619,16 +654,7 @@ class ReportOptions(BaseOptions):
                 self.pad_drill_real_ec = min(dr_ec, self.pad_drill_real_ec)
                 oar_t = min(oar_x, oar_y)
                 oar_ec_t = min(oar_ec_x, oar_ec_y)
-                if oar_t > 0:
-                    self.oar_pads = min(self.oar_pads, oar_t)
-                    self.oar_pads_ec = min(self.oar_pads_ec, oar_ec_t)
-                    if oar_t < min_oar:
-                        logger.warning(W_WRONGOAR+"Really small OAR detected ({} mm) for pad {}".
-                                       format(to_mm(oar_t, 4), get_pad_info(pad)))
-                elif oar_t < 0:
-                    logger.warning(W_WRONGOAR+"Negative OAR detected for pad "+get_pad_info(pad))
-                elif oar_t == 0 and is_pth:
-                    logger.warning(W_WRONGOAR+"Plated pad without copper "+get_pad_info(pad))
+                self.analyze_oar(oar_t, oar_ec_t, is_pth, min_oar, pad, dr_x_real, dr_y_real, pad_sz, dr)
         self._vias_m = sorted(self._vias.keys())
         self._vias_ec_m = sorted(self._vias_ec.keys())
         # Via Pad size
@@ -730,9 +756,28 @@ class ReportOptions(BaseOptions):
         logger.debug('- Result `{}`'.format(res))
         return res
 
+    def get_kicad_vars(self):
+        vars = {}
+        vars['KICAD_VERSION'] = 'KiCad E.D.A. '+GS.kicad_version+' + KiBot v'+__version__
+        GS.load_pcb_title_block()
+        for num in range(9):
+            vars['COMMENT'+str(num+1)] = GS.pcb_com[num]
+        vars['COMPANY'] = GS.pcb_comp
+        vars['ISSUE_DATE'] = GS.pcb_date
+        vars['REVISION'] = GS.pcb_rev
+        vars['TITLE'] = GS.pcb_title
+        vars['FILENAME'] = GS.pcb_basename+'.kicad_pcb'
+        return vars
+
     def do_template(self, template_file, output_file):
         text = ''
         logger.debug("Report template: `{}`".format(template_file))
+        # Collect the thing we could expand
+        defined = {}
+        defined.update(os.environ)
+        defined.update(self.get_kicad_vars())
+        defined.update(GS.load_pro_variables())
+        defined.update(self.__dict__)
         with open(template_file, "rt") as f:
             skip_next = False
             for line in f:
@@ -757,7 +802,7 @@ class ReportOptions(BaseOptions):
                             raise KiPlotConfigurationError("Unknown context: `{}`".format(context))
                 if not done:
                     # Just replace using any data member (_* excluded)
-                    line = self.do_replacements(line, self.__dict__)
+                    line = self.do_replacements(line, defined)
                 text += line
         logger.debug("Report output: `{}`".format(output_file))
         if self.to_ascii:
@@ -863,7 +908,9 @@ class ReportOptions(BaseOptions):
 class Report(BaseOutput):  # noqa: F821
     """ Design report
         Generates a report about the design.
-        Mainly oriented to be sent to the manufacturer or check PCB details. """
+        Mainly oriented to be sent to the manufacturer or check PCB details.
+        You can expand internal values, KiCad text variables and environment
+        variables using `${VARIABLE}` """
     def __init__(self):
         super().__init__()
         with document:
